==> ide-impl-193.5233.109-sources.jar: com/intellij/internal/performance/Latenciometer.kt <==
// Copyright 2000-2019 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.internal.performance

import com.intellij.openapi.application.ApplicationManager
import com.intellij.openapi.editor.Editor
import com.intellij.openapi.editor.actionSystem.LatencyListener
import com.intellij.openapi.editor.actionSystem.LatencyRecorder
import com.intellij.openapi.editor.impl.EditorImpl
import com.intellij.openapi.fileEditor.FileDocumentManager
import gnu.trove.TIntArrayList

/**
 * @author yole
 */

class LatencyRecorderImpl : LatencyRecorder {
  override fun recordLatencyAwareAction(editor: Editor, actionId: String, timestampMs: Long) {
    (editor as? EditorImpl)?.recordLatencyAwareAction(actionId, timestampMs)
  }
}

class LatencyRecord {
  var totalLatency: Long = 0L
  var maxLatency: Int = 0
  val samples = TIntArrayList()
  var samplesSorted = false

  fun update(latencyInMS: Int) {
    samplesSorted = false
    samples.add(latencyInMS)
    totalLatency += latencyInMS
    if (latencyInMS > maxLatency) {
      maxLatency = latencyInMS
    }
  }

  val averageLatency: Long get() = totalLatency / samples.size()

  fun percentile(n: Int): Int {
    if (!samplesSorted) {
      samples.sort()
      samplesSorted = true
    }
    val index = (samples.size() * n / 100).coerceAtMost(samples.size() - 1)
    return samples[index]
  }
}

data class LatencyDistributionRecordKey(val name: String) {
  var details: String? = null
}

class LatencyDistributionRecord(val key: LatencyDistributionRecordKey) {
  val totalLatency: LatencyRecord = LatencyRecord()
  val actionLatencyRecords: MutableMap<String, LatencyRecord> = mutableMapOf()

  fun update(action: String, latencyInMS: Int) {
    totalLatency.update(latencyInMS)
    actionLatencyRecords.getOrPut(action) { LatencyRecord() }.update(latencyInMS)
  }
}

val latencyMap: MutableMap<LatencyDistributionRecordKey, LatencyDistributionRecord> = mutableMapOf()

var currentLatencyRecordKey: LatencyDistributionRecordKey? = null

val latencyRecorderProperties: MutableMap<String, String> = mutableMapOf()

class LatenciometerListener : LatencyListener {

  init {
    ApplicationManager.getApplication().messageBus.connect().subscribe(LatencyListener.TOPIC, this)
  }

  override fun recordTypingLatency(editor: Editor, action: String, latencyInMS: Long) {
    val key = currentLatencyRecordKey ?: run {
      val fileType = FileDocumentManager.getInstance().getFile(editor.document)?.fileType ?: return
      LatencyDistributionRecordKey(fileType.name)
    }
    val latencyRecord = latencyMap.getOrPut(key) {
      LatencyDistributionRecord(key)
    }
    latencyRecord.update(getActionKey(action), latencyInMS.toInt())
  }
}

fun getActionKey(action: String): String =
  if (action.length == 1) {
    when(action[0]) {
      in 'A'..'Z', in 'a'..'z', in '0'..'9' -> "Letter"
      ' ' -> "Space"
      '\n' -> "Enter"
      else -> action
    }
  }
  else action

==> ide-impl-202.6397.98-sources.jar: com/intellij/internal/performance/Latenciometer.kt <==
// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.internal.performance

import com.intellij.openapi.editor.Editor
import com.intellij.openapi.editor.actionSystem.LatencyListener
import com.intellij.openapi.editor.actionSystem.LatencyRecorder
import com.intellij.openapi.editor.impl.EditorImpl
import com.intellij.openapi.fileEditor.FileDocumentManager
import gnu.trove.TIntArrayList

private class LatencyRecorderImpl : LatencyRecorder {
  override fun recordLatencyAwareAction(editor: Editor, actionId: String, timestampMs: Long) {
    (editor as? EditorImpl)?.recordLatencyAwareAction(actionId, timestampMs)
  }
}

class LatencyRecord {
  var totalLatency: Long = 0L
  var maxLatency: Int = 0
  val samples = TIntArrayList()
  var samplesSorted = false

  fun update(latencyInMS: Int) {
    samplesSorted = false
    samples.add(latencyInMS)
    totalLatency += latencyInMS
    if (latencyInMS > maxLatency) {
      maxLatency = latencyInMS
    }
  }

  val averageLatency: Long
    get() = totalLatency / samples.size()

  fun percentile(n: Int): Int {
    if (!samplesSorted) {
      samples.sort()
      samplesSorted = true
    }
    val index = (samples.size() * n / 100).coerceAtMost(samples.size() - 1)
    return samples[index]
  }
}

data class LatencyDistributionRecordKey(val name: String) {
  var details: String? = null
}

class LatencyDistributionRecord(val key: LatencyDistributionRecordKey) {
  val totalLatency: LatencyRecord = LatencyRecord()
  val actionLatencyRecords: MutableMap<String, LatencyRecord> = mutableMapOf()

  fun update(action: String, latencyInMS: Int) {
    totalLatency.update(latencyInMS)
    actionLatencyRecords.getOrPut(action) { LatencyRecord() }.update(latencyInMS)
  }
}

val latencyMap: MutableMap<LatencyDistributionRecordKey, LatencyDistributionRecord> = mutableMapOf()

var currentLatencyRecordKey: LatencyDistributionRecordKey? = null

val latencyRecorderProperties: MutableMap<String, String> = mutableMapOf()

private class LatenciometerListener : LatencyListener {
  override fun recordTypingLatency(editor: Editor, action: String, latencyInMS: Long) {
    val key = currentLatencyRecordKey ?: run {
      val fileType = FileDocumentManager.getInstance().getFile(editor.document)?.fileType ?: return
      LatencyDistributionRecordKey(fileType.name)
    }
    val latencyRecord = latencyMap.getOrPut(key) {
      LatencyDistributionRecord(key)
    }
    latencyRecord.update(getActionKey(action), latencyInMS.toInt())
  }
}

private fun getActionKey(action: String): String {
  return if (action.length == 1) {
    when(action[0]) {
      in 'A'..'Z', in 'a'..'z', in '0'..'9' -> "Letter"
      ' ' -> "Space"
      '\n' -> "Enter"
      else -> action
    }
  }
  else action
}
